@@ -8,34 +8,28 @@
 module Command.InitRemote where
 
 import qualified Data.Map as M
-import Control.Monad (when)
-import Control.Monad.State (liftIO)
-import Data.Maybe
-import Data.String.Utils
 
+import Common.Annex
 import Command
 import qualified Remote
-import qualified RemoteLog
+import qualified Logs.Remote
 import qualified Types.Remote as R
-import Types
-import UUID
-import Messages
+import Annex.UUID
 
-command :: [Command]
-command = [repoCommand "initremote"
-	(paramPair paramName $
-		paramOptional $ paramRepeating paramKeyValue) seek
-	"sets up a special (non-git) remote"]
+def :: [Command]
+def = [command "initremote"
+	(paramPair paramName $ paramOptional $ paramRepeating paramKeyValue)
+	seek "sets up a special (non-git) remote"]
 
 seek :: [CommandSeek]
 seek = [withWords start]
 
-start :: CommandStartWords
+start :: [String] -> CommandStart
 start ws = do
 	when (null ws) needname
 
 	(u, c) <- findByName name
-	let fullconfig = M.union config c	
+	let fullconfig = config `M.union` c	
 	t <- findType fullconfig
 
 	showStart "initremote" name
@@ -43,7 +37,7 @@ start ws = do
 
 	where
 		name = head ws
-		config = RemoteLog.keyValToConfig $ tail ws
+		config = Logs.Remote.keyValToConfig $ tail ws
 		needname = do
 			let err s = error $ "Specify a name for the remote. " ++ s
 			names <- remoteNames
@@ -59,13 +53,13 @@ perform t u c = do
 
 cleanup :: UUID -> R.RemoteConfig -> CommandCleanup
 cleanup u c = do
-	RemoteLog.configSet u c
+	Logs.Remote.configSet u c
         return True
 
 {- Look up existing remote's UUID and config by name, or generate a new one -}
 findByName :: String -> Annex (UUID, R.RemoteConfig)
 findByName name = do
-	m <- RemoteLog.readRemoteLog
+	m <- Logs.Remote.readRemoteLog
 	maybe generate return $ findByName' name m
 	where
 		generate = do
@@ -73,7 +67,9 @@ findByName name = do
 			return (uuid, M.insert nameKey name M.empty)
 
 findByName' :: String ->  M.Map UUID R.RemoteConfig -> Maybe (UUID, R.RemoteConfig)
-findByName' n m = if null matches then Nothing else Just $ head matches
+findByName' n m
+	| null matches = Nothing
+	| otherwise = Just $ head matches
 	where
 		matches = filter (matching . snd) $ M.toList m
 		matching c = case M.lookup nameKey c of
@@ -84,7 +80,7 @@ findByName' n m = if null matches then Nothing else Just $ head matches
 
 remoteNames :: Annex [String]
 remoteNames = do
-	m <- RemoteLog.readRemoteLog
+	m <- Logs.Remote.readRemoteLog
 	return $ mapMaybe (M.lookup nameKey . snd) $ M.toList m
 
 {- find the specified remote type -}
