@@ -8,47 +8,38 @@
 module Command.Status where
 
 import Control.Monad.State
-import Data.Maybe
-import System.IO
-import Data.List
 import qualified Data.Map as M
+import qualified Data.Set as S
+import Data.Set (Set)
 
-import qualified Annex
+import Common.Annex
 import qualified Types.Backend as B
 import qualified Types.Remote as R
 import qualified Remote
 import qualified Command.Unused
 import qualified Git
 import Command
-import Types
 import Utility.DataUnits
-import Content
+import Annex.Content
 import Types.Key
-import Locations
 import Backend
+import Logs.UUID
+import Remote
 
 -- a named computation that produces a statistic
 type Stat = StatState (Maybe (String, StatState String))
 
 -- cached info that multiple Stats may need
 data StatInfo = StatInfo
-	{ keysPresentCache :: Maybe (SizeList Key)
-	, keysReferencedCache :: Maybe (SizeList Key)
+	{ keysPresentCache :: Maybe (Set Key)
+	, keysReferencedCache :: Maybe (Set Key)
 	}
 
 -- a state monad for running Stats in
 type StatState = StateT StatInfo Annex
 
--- a list with a known length
--- (Integer is used for the length to avoid
--- blowing up if someone annexed billions of files..)
-type SizeList a = ([a], Integer)
-
-sizeList :: [a] -> SizeList a
-sizeList l = (l, genericLength l)
-
-command :: [Command]
-command = [repoCommand "status" paramNothing seek
+def :: [Command]
+def = [command "status" paramNothing seek
 	"shows status information about the annex"]
 
 seek :: [CommandSeek]
@@ -57,27 +48,23 @@ seek = [withNothing start]
 {- Order is significant. Less expensive operations, and operations
  - that share data go together.
  -}
-faststats :: [Stat]
-faststats = 
+stats :: [Stat]
+stats = 
 	[ supported_backends
 	, supported_remote_types
+	, remote_list
 	, tmp_size
 	, bad_data_size
-	]
-slowstats :: [Stat]
-slowstats =
-	[ local_annex_keys
+	, local_annex_keys
 	, local_annex_size
-	, total_annex_keys
-	, total_annex_size
+	, visible_annex_keys
+	, visible_annex_size
 	, backend_usage
 	]
 
-start :: CommandStartNothing
+start :: CommandStart
 start = do
-	fast <- Annex.getState Annex.fast
-	let todo = if fast then faststats else faststats ++ slowstats
-	evalStateT (mapM_ showStat todo) (StatInfo Nothing Nothing)
+	evalStateT (mapM_ showStat stats) (StatInfo Nothing Nothing)
 	stop
 
 stat :: String -> StatState String -> Stat
@@ -103,21 +90,26 @@ supported_remote_types :: Stat
 supported_remote_types = stat "supported remote types" $
 	return $ unwords $ map R.typename Remote.remoteTypes
 
+remote_list :: Stat
+remote_list = stat "known repositories" $ lift $ do
+	s <- prettyPrintUUIDs "repos" =<< M.keys <$> uuidMap
+	return $ '\n':init s
+
 local_annex_size :: Stat
 local_annex_size = stat "local annex size" $
-	cachedKeysPresent >>= keySizeSum
-
-total_annex_size :: Stat
-total_annex_size = stat "total annex size" $
-	cachedKeysReferenced >>= keySizeSum
+	keySizeSum <$> cachedKeysPresent
 
 local_annex_keys :: Stat
-local_annex_keys = stat "local annex keys" $ 
-	return . show . snd =<< cachedKeysPresent
+local_annex_keys = stat "local annex keys" $
+	show . S.size <$> cachedKeysPresent
 
-total_annex_keys :: Stat
-total_annex_keys = stat "total annex keys" $
-	return . show . snd =<< cachedKeysReferenced
+visible_annex_size :: Stat
+visible_annex_size = stat "visible annex size" $
+	keySizeSum <$> cachedKeysReferenced
+
+visible_annex_keys :: Stat
+visible_annex_keys = stat "visible annex keys" $
+	show . S.size <$> cachedKeysReferenced
 
 tmp_size :: Stat
 tmp_size = staleSize "temporary directory size" gitAnnexTmpDir
@@ -126,10 +118,9 @@ bad_data_size :: Stat
 bad_data_size = staleSize "bad keys size" gitAnnexBadDir
 
 backend_usage :: Stat
-backend_usage = stat "backend usage" $
-	return . usage =<< cachedKeysReferenced
+backend_usage = stat "backend usage" $ usage <$> cachedKeysReferenced <*> cachedKeysPresent
 	where
-		usage (ks, _) = pp "" $ sort $ map swap $ splits ks
+		usage a b = pp "" $ reverse . sort $ map swap $ splits $ S.toList $ S.union a b
 		splits :: [Key] -> [(String, Integer)]
 		splits ks = M.toList $ M.fromListWith (+) $ map tcount ks
 		tcount k = (keyBackendName k, 1)
@@ -137,40 +128,37 @@ backend_usage = stat "backend usage" $
 		pp c [] = c
 		pp c ((n, b):xs) = "\n\t" ++ b ++ ": " ++ show n ++ pp c xs
 
-
-cachedKeysPresent :: StatState (SizeList Key)
+cachedKeysPresent :: StatState (Set Key)
 cachedKeysPresent = do
 	s <- get
 	case keysPresentCache s of
 		Just v -> return v
 		Nothing -> do
-			keys <- lift getKeysPresent
-			let v = sizeList keys
-			put s { keysPresentCache = Just v }
-			return v
+			keys <- S.fromList <$> lift getKeysPresent
+			put s { keysPresentCache = Just keys }
+			return keys
 
-cachedKeysReferenced :: StatState (SizeList Key)
+cachedKeysReferenced :: StatState (Set Key)
 cachedKeysReferenced = do
 	s <- get
 	case keysReferencedCache s of
 		Just v -> return v
 		Nothing -> do
-			keys <- lift Command.Unused.getKeysReferenced
-			-- A given key may be referenced repeatedly.
-			-- nub does not seem too slow (yet)..
-			let v = sizeList $ nub keys
-			put s { keysReferencedCache = Just v }
-			return v
+			keys <- S.fromList <$> lift Command.Unused.getKeysReferenced
+			put s { keysReferencedCache = Just keys }
+			return keys
 
-keySizeSum :: SizeList Key -> StatState String
-keySizeSum (keys, len) = do
-	let knownsizes = mapMaybe keySize keys
-	let total = roughSize storageUnits False $ sum knownsizes
-	let missing = len - genericLength knownsizes
-	return $ total ++
-		if missing > 0
-			then aside $ "but " ++ show missing ++ " keys have unknown size"
-			else ""
+keySizeSum :: Set Key -> String
+keySizeSum s = total ++ missingnote
+	where
+		knownsizes = mapMaybe keySize $ S.toList s
+		total = roughSize storageUnits False $ sum knownsizes
+		missing = S.size s - genericLength knownsizes
+		missingnote
+			| missing == 0 = ""
+			| otherwise = aside $
+				"+ " ++ show missing ++
+				" keys of unknown size"
 
 staleSize :: String -> (Git.Repo -> FilePath) -> Stat
 staleSize label dirspec = do
@@ -178,8 +166,8 @@ staleSize label dirspec = do
 	if null keys
 		then nostat
 		else stat label $ do
-			s <- keySizeSum $ sizeList keys
+			let s = keySizeSum $ S.fromList keys
 			return $ s ++ aside "clean up with git-annex unused"
 
 aside :: String -> String
-aside s = "\t(" ++ s ++ ")"
+aside s = " (" ++ s ++ ")"
