@@ -8,17 +8,15 @@
 module GitAnnex where
 
 import System.Console.GetOpt
-import Control.Monad.State (liftIO)
 
+import Common.Annex
 import qualified Git
 import CmdLine
 import Command
-import Options
-import Utility
-import Types
 import Types.TrustLevel
 import qualified Annex
 import qualified Remote
+import qualified Limit
 
 import qualified Command.Add
 import qualified Command.Unannex
@@ -28,7 +26,7 @@ import qualified Command.Copy
 import qualified Command.Get
 import qualified Command.FromKey
 import qualified Command.DropKey
-import qualified Command.SetKey
+import qualified Command.Reinject
 import qualified Command.Fix
 import qualified Command.Init
 import qualified Command.Describe
@@ -55,50 +53,46 @@ import qualified Command.Version
 
 cmds :: [Command]
 cmds = concat
-	[ Command.Add.command
-	, Command.Get.command
-	, Command.Drop.command
-	, Command.Move.command
-	, Command.Copy.command
-	, Command.Unlock.command
-	, Command.Lock.command
-	, Command.Init.command
-	, Command.Describe.command
-	, Command.InitRemote.command
-	, Command.Unannex.command
-	, Command.Uninit.command
-	, Command.PreCommit.command
-	, Command.Trust.command
-	, Command.Untrust.command
-	, Command.Semitrust.command
-	, Command.AddUrl.command
-	, Command.FromKey.command
-	, Command.DropKey.command
-	, Command.SetKey.command
-	, Command.Fix.command
-	, Command.Fsck.command
-	, Command.Unused.command
-	, Command.DropUnused.command
-	, Command.Find.command
-	, Command.Whereis.command
-	, Command.Merge.command
-	, Command.Status.command
-	, Command.Migrate.command
-	, Command.Map.command
-	, Command.Upgrade.command
-	, Command.Version.command
+	[ Command.Add.def
+	, Command.Get.def
+	, Command.Drop.def
+	, Command.Move.def
+	, Command.Copy.def
+	, Command.Unlock.def
+	, Command.Lock.def
+	, Command.Init.def
+	, Command.Describe.def
+	, Command.InitRemote.def
+	, Command.Reinject.def
+	, Command.Unannex.def
+	, Command.Uninit.def
+	, Command.PreCommit.def
+	, Command.Trust.def
+	, Command.Untrust.def
+	, Command.Semitrust.def
+	, Command.AddUrl.def
+	, Command.FromKey.def
+	, Command.DropKey.def
+	, Command.Fix.def
+	, Command.Fsck.def
+	, Command.Unused.def
+	, Command.DropUnused.def
+	, Command.Find.def
+	, Command.Whereis.def
+	, Command.Merge.def
+	, Command.Status.def
+	, Command.Migrate.def
+	, Command.Map.def
+	, Command.Upgrade.def
+	, Command.Version.def
 	]
 
 options :: [Option]
 options = commonOptions ++
-	[ Option ['k'] ["key"] (ReqArg setkey paramKey)
-		"specify a key to use"
-	, Option ['t'] ["to"] (ReqArg setto paramRemote)
+	[ Option ['t'] ["to"] (ReqArg setto paramRemote)
 		"specify to where to transfer content"
 	, Option ['f'] ["from"] (ReqArg setfrom paramRemote)
 		"specify from where to transfer content"
-	, Option ['x'] ["exclude"] (ReqArg addexclude paramGlob)
-		"skip files matching the glob pattern"
 	, Option ['N'] ["numcopies"] (ReqArg setnumcopies paramNumber)
 		"override default number of copies"
 	, Option [] ["trust"] (ReqArg (Remote.forceTrust Trusted) paramRemote)
@@ -109,18 +103,21 @@ options = commonOptions ++
 		"override trust setting to untrusted"
 	, Option ['c'] ["config"] (ReqArg setgitconfig "NAME=VALUE")
 		"override git configuration setting"
-	]
+	, Option ['x'] ["exclude"] (ReqArg Limit.addExclude paramGlob)
+		"skip files matching the glob pattern"
+	, Option ['i'] ["in"] (ReqArg Limit.addIn paramRemote)
+		"skip files not present in a remote"
+	, Option ['C'] ["copies"] (ReqArg Limit.addCopies paramNumber)
+		"skip files with fewer copies"
+	] ++ matcherOptions
 	where
 		setto v = Annex.changeState $ \s -> s { Annex.toremote = Just v }
 		setfrom v = Annex.changeState $ \s -> s { Annex.fromremote = Just v }
-		addexclude v = Annex.changeState $ \s -> s { Annex.exclude = v:Annex.exclude s }
 		setnumcopies v = Annex.changeState $ \s -> s {Annex.forcenumcopies = readMaybe v }
-		setkey v = Annex.changeState $ \s -> s { Annex.defaultkey = Just v }
 		setgitconfig :: String -> Annex ()
 		setgitconfig v = do
-			g <- Annex.gitRepo
-			g' <- liftIO $ Git.configStore g v
-			Annex.changeState $ \s -> s { Annex.repo = g' }
+			newg <- inRepo $ Git.configStore v
+			Annex.changeState $ \s -> s { Annex.repo = newg }
 
 header :: String
 header = "Usage: git-annex command [option ..]"
