@@ -16,7 +16,6 @@ module Remote (
 	hasKeyCheap,
 
 	remoteTypes,
-	genList,
 	byName,
 	prettyPrintUUIDs,
 	remotesWithUUID,
@@ -26,24 +25,23 @@ module Remote (
 	nameToUUID,
 	showTriedRemotes,
 	showLocations,
-	forceTrust
+	forceTrust,
+	logStatus
 ) where
 
-import Control.Monad (filterM, liftM2)
-import Data.List
 import qualified Data.Map as M
-import Data.String.Utils
-import Data.Maybe
+import Text.JSON
+import Text.JSON.Generic
 
-import Types
+import Common.Annex
 import Types.Remote
-import UUID
 import qualified Annex
 import Config
-import Trust
-import LocationLog
-import Messages
-import RemoteLog
+import Annex.UUID
+import Logs.UUID
+import Logs.Trust
+import Logs.Location
+import Logs.Remote
 
 import qualified Remote.Git
 import qualified Remote.S3
@@ -80,13 +78,13 @@ genList = do
 	where
 		process m t = 
 			enumerate t >>=
-			filterM remoteNotIgnored >>=
 			mapM (gen m t)
 		gen m t r = do
-			u <- getUUID r
+			u <- getRepoUUID r
 			generate t r u (M.lookup u m)
 
-{- Looks up a remote by name. (Or by UUID.) -}
+{- Looks up a remote by name. (Or by UUID.) Only finds currently configured
+ - git remotes. -}
 byName :: String -> Annex (Remote Annex)
 byName n = do
 	res <- byName' n
@@ -102,39 +100,62 @@ byName' n = do
 		then return $ Left $ "there is no git remote named \"" ++ n ++ "\""
 		else return $ Right $ head match
 	where
-		matching r = n == name r || n == uuid r
+		matching r = n == name r || toUUID n == uuid r
 
 {- Looks up a remote by name (or by UUID, or even by description),
- - and returns its UUID. -}
+ - and returns its UUID. Finds even remotes that are not configured in
+ - .git/config. -}
 nameToUUID :: String -> Annex UUID
-nameToUUID "." = getUUID =<< Annex.gitRepo -- special case for current repo
-nameToUUID n = do
-	res <- byName' n
-	case res of
-		Left e -> return . fromMaybe (error e) =<< byDescription
-		Right r -> return $ uuid r
+nameToUUID "." = getUUID -- special case for current repo
+nameToUUID n = byName' n >>= go
 	where
-		byDescription = return . M.lookup n . invertMap =<< uuidMap
-		invertMap = M.fromList . map swap . M.toList
+		go (Right r) = return $ uuid r
+		go (Left e) = fromMaybe (error e) <$> bydescription
+		bydescription = do
+			m <- uuidMap
+			case M.lookup n $ transform swap m of
+				Just u -> return $ Just u
+				Nothing -> return $ byuuid m
+		byuuid m = M.lookup (toUUID n) $ transform double m
+		transform a = M.fromList . map a . M.toList
 		swap (a, b) = (b, a)
+		double (a, _) = (a, a)
 
-{- Pretty-prints a list of UUIDs of remotes. -}
-prettyPrintUUIDs :: [UUID] -> Annex String
-prettyPrintUUIDs uuids = do
-	here <- getUUID =<< Annex.gitRepo
-	-- Show descriptions from the uuid log, falling back to remote names,
-	-- as some remotes may not be in the uuid log
-	m <- liftM2 M.union uuidMap $
-		return . M.fromList . map (\r -> (uuid r, name r)) =<< genList
-	return $ unwords $ map (\u -> "\t" ++ prettify m u here ++ "\n") uuids
+{- Pretty-prints a list of UUIDs of remotes, for human display.
+ -
+ - Shows descriptions from the uuid log, falling back to remote names,
+ - as some remotes may not be in the uuid log.
+ -
+ - When JSON is enabled, also generates a machine-readable description
+ - of the UUIDs. -}
+prettyPrintUUIDs :: String -> [UUID] -> Annex String
+prettyPrintUUIDs desc uuids = do
+	hereu <- getUUID
+	m <- M.unionWith addname <$> uuidMap <*> remoteMap
+	maybeShowJSON [(desc, map (jsonify m hereu) uuids)]
+	return $ unwords $ map (\u -> "\t" ++ prettify m hereu u ++ "\n") uuids
 	where
-		prettify m u here = base ++ ishere
-			where
-				base = if not $ null $ findlog m u
-					then u ++ "  -- " ++ findlog m u
-					else u
-				ishere = if here == u then " <-- here" else ""
+		addname d n
+			| d == n = d
+			| null d = n
+			| otherwise = n ++ " (" ++ d ++ ")"
+		remoteMap = M.fromList . map (\r -> (uuid r, name r)) <$> genList
 		findlog m u = M.findWithDefault "" u m
+		prettify m hereu u
+			| not (null d) = fromUUID u ++ " -- " ++ d
+			| otherwise = fromUUID u
+			where
+				ishere = hereu == u
+				n = findlog m u
+				d
+					| null n && ishere = "here"
+					| ishere = addname n "here"
+					| otherwise = n
+		jsonify m hereu u = toJSObject
+			[ ("uuid", toJSON $ fromUUID u)
+			, ("description", toJSON $ findlog m u)
+			, ("here", toJSON $ hereu == u)
+			]
 
 {- Filters a list of remotes to ones that have the listed uuids. -}
 remotesWithUUID :: [Remote Annex] -> [UUID] -> [Remote Annex]
@@ -144,12 +165,12 @@ remotesWithUUID rs us = filter (\r -> uuid r `elem` us) rs
 remotesWithoutUUID :: [Remote Annex] -> [UUID] -> [Remote Annex]
 remotesWithoutUUID rs us = filter (\r -> uuid r `notElem` us) rs
 
-{- Cost ordered lists of remotes that the LocationLog indicate may have a key.
+{- Cost ordered lists of remotes that the Logs.Location indicate may have a key.
  -}
 keyPossibilities :: Key -> Annex [Remote Annex]
-keyPossibilities key = return . fst =<< keyPossibilities' False key
+keyPossibilities key = fst <$> keyPossibilities' False key
 
-{- Cost ordered lists of remotes that the LocationLog indicate may have a key.
+{- Cost ordered lists of remotes that the Logs.Location indicate may have a key.
  -
  - Also returns a list of UUIDs that are trusted to have the key
  - (some may not have configured remotes).
@@ -159,8 +180,7 @@ keyPossibilitiesTrusted = keyPossibilities' True
 
 keyPossibilities' :: Bool -> Key -> Annex ([Remote Annex], [UUID])
 keyPossibilities' withtrusted key = do
-	g <- Annex.gitRepo
-	u <- getUUID g
+	u <- getUUID
 	trusted <- if withtrusted then trustGet Trusted else return []
 
 	-- get uuids of all remotes that are recorded to have the key
@@ -168,10 +188,10 @@ keyPossibilities' withtrusted key = do
 	let validuuids = filter (/= u) uuids
 
 	-- note that validuuids is assumed to not have dups
-	let validtrusteduuids = intersect validuuids trusted
+	let validtrusteduuids = validuuids `intersect` trusted
 
 	-- remotes that match uuids that have the key
-	allremotes <- genList
+	allremotes <- filterM (repoNotIgnored . repo) =<< genList
 	let validremotes = remotesWithUUID allremotes validuuids
 
 	return (sort validremotes, validtrusteduuids)
@@ -179,14 +199,13 @@ keyPossibilities' withtrusted key = do
 {- Displays known locations of a key. -}
 showLocations :: Key -> [UUID] -> Annex ()
 showLocations key exclude = do
-	g <- Annex.gitRepo
-	u <- getUUID g
+	u <- getUUID
 	uuids <- keyLocations key
 	untrusteduuids <- trustGet UnTrusted
 	let uuidswanted = filteruuids uuids (u:exclude++untrusteduuids) 
 	let uuidsskipped = filteruuids uuids (u:exclude++uuidswanted)
-	ppuuidswanted <- Remote.prettyPrintUUIDs uuidswanted
-	ppuuidsskipped <- Remote.prettyPrintUUIDs uuidsskipped
+	ppuuidswanted <- Remote.prettyPrintUUIDs "wanted" uuidswanted
+	ppuuidsskipped <- Remote.prettyPrintUUIDs "skipped" uuidsskipped
 	showLongNote $ message ppuuidswanted ppuuidsskipped
 	where
 		filteruuids l x = filter (`notElem` x) l
@@ -206,3 +225,12 @@ forceTrust level remotename = do
 	r <- nameToUUID remotename
 	Annex.changeState $ \s ->
 		s { Annex.forcetrust = (r, level):Annex.forcetrust s }
+
+{- Used to log a change in a remote's having a key. The change is logged
+ - in the local repo, not on the remote. The process of transferring the
+ - key to the remote, or removing the key from it *may* log the change
+ - on the remote, but this cannot always be relied on. -}
+logStatus :: Remote Annex -> Key -> Bool -> Annex ()
+logStatus remote key present = logChange key (uuid remote) status
+	where
+		status = if present then InfoPresent else InfoMissing
