@@ -1,4 +1,4 @@
-{- git-annex file copying with rsync
+{- file copying with rsync
  -
  - Copyright 2010 Joey Hess <joey@kitenet.net>
  -
@@ -9,7 +9,7 @@ module Utility.RsyncFile where
 
 import Data.String.Utils
 
-import Utility
+import Utility.SafeCommand
 
 {- Generates parameters to make rsync use a specified command as its remote
  - shell. -}
@@ -35,6 +35,8 @@ rsyncServerParams =
 	[ Param "--server"
 	-- preserve permissions
 	, Param "-p"
+	-- preserve timestamps
+	, Param "-t"
 	-- allow resuming of transfers of big files
 	, Param "--inplace"
 	-- other options rsync normally uses in server mode
