@@ -10,6 +10,4 @@ import System.Environment
 import GitAnnex
 
 main :: IO ()
-main = do
-	args <- getArgs
-	run args
+main = run =<< getArgs
